import math


def remove_directory(directory):
    if not directory.is_dir():
        return

    for item in directory.iterdir():
        if item.is_dir():
            remove_directory(item)
        else:
            item.unlink()

    directory.rmdir()


def get_files_in_directory(directory):

    files = set()

    if not directory.is_dir():
        return files

    for item in directory.iterdir():
        if item.is_dir():
            files.update(get_files_in_directory(item))
        elif item.is_file():
            files.add(item.resolve())

    return files


def get_deep_attribute(obj, attribute_path):
    if type(attribute_path) == str:
        return get_deep_attribute(obj, attribute_path.split('.'))

    current_object = obj
    for attribute_name in attribute_path:
        current_object = getattr(current_object, attribute_name)

    return current_object


def get_exception_with_traceback(exception):
    import traceback

    error_message = unicode(exception)

    # if the return value is an exception, add it's stack trace to the error message
    if hasattr(exception, "__traceback__"):
        stack_trace = traceback.format_tb(exception.__traceback__)
        for line in stack_trace:
            error_message = error_message + unicode(line)

    return error_message


def get_unique_id(objects, attribute_name='id'):
    """ Returns an integer ID that has not been used by any other object in the provided list.

    :param objects: a list of existing objects that may or may not have an id attribute
    :type objects: list
    :param attribute_name: The name of the attribute or getter method containing the id
    :type attribute_name: unicode
    :return: the new unique id
    :rtype: int
    """

    used_ids = []

    for object in objects:

        if not hasattr(object, attribute_name):
            continue

        id = getattr(object, attribute_name)
        if callable(id):
            id = id()

        if not isinstance(id, int):
            continue

        used_ids.append(id)

    # find a number which has not been used yet
    unused_id = 0

    while unused_id in used_ids:
        unused_id = unused_id + 1

    return unused_id


def create_unique_object_name(objects, attribute_name='name', name_prefix = ""):
    """ Creates a unique name of the form [name_prefix][x] where x is some unique positive integer.

    :param objects: the existing list of objects that this new name must not collide with. Objects in said list may or
                    may not have .name attributes or follow the provided naming scheme
    :type objects: list
    :param attribute_name: The name of the attribute or getter method which contains the objects name
    :type attribute_name: unicode
    :param name_prefix: a optional prefix to append before the unique number
    :type name_prefix: unicode
    :return: the unique name
    :rtype: unicode
    """

    prefix_length = len(name_prefix)

    # compile a list of used numeric integer suffixes
    used_suffixes = []
    for element in objects:

        if not hasattr(element, attribute_name):
            continue

        name = getattr(element, attribute_name)

        if callable(name):
            name = name()

        if not isinstance(name, unicode):
            continue

        if not name.startswith(name_prefix):
            continue

        element_suffix = name[prefix_length:]
        if element_suffix.isdigit():
            try:
                element_suffix = int(element_suffix)

                if element_suffix >= 0:
                    used_suffixes.append(element_suffix)

            except ValueError:
                pass

    # remove duplicates
    used_suffixes = list(set(used_suffixes))

    # sort
    used_suffixes.sort()

    # find a number which has not been used yet
    i = 0
    while i in used_suffixes:
        i = i+1

    return name_prefix + unicode(i)


def get_desktop_path():
    import os

    if os.name == 'nt':
        return os.path.join(os.path.join(os.environ['USERPROFILE']), 'Desktop')
    elif os.name == 'posix':
        return os.path.join(os.path.join(os.path.expanduser('~')), 'Desktop')
    else:
        return ""


def bytes_to_human(bytes):
    size_name = ("B", "KB", "MB", "GB", "TB", "PB", "EB", "ZB", "YB")

    if bytes == 0:
        return "0 B"

    bytes = int(bytes)
    index = int(math.floor(math.log(bytes, 1000)))
    power = math.pow(1000, index)
    size = round(bytes / power, 2)
    return "{} {}".format(size, size_name[index])
