__all__ = 'PreflightWindow'

from PySide2 import QtCore
from PySide2 import QtWidgets

from gm_qt_plugin import qt_constants
from gm_qt_plugin.qt_plugin_accessor import QtPluginAccessor
from gm_qt_plugin.qt_model_file_tree import FileTreeModel


class PreflightWindow(QtWidgets.QDialog, QtPluginAccessor):

    INITIAL_WINDOW_WIDTH = 800
    INITIAL_WINDOW_HEIGHT = 700

    MINIMUM_WINDOW_WIDTH = 600
    MINIMUM_WINDOW_HEIGHT = 600

    UPLOAD_AND_SUBMIT = u"Upload and Submit"
    UPLOAD = u"Upload"
    SUBMIT = u"Submit"

    UPLOAD_AND_SUBMIT_TOOL_TIP = u"Tell Envoy to upload the above selected files and submit your job once all files have been uploaded."
    UPLOAD_TOOL_TIP = u"Tell Envoy to upload the above selected files."
    SUBMIT_TOOL_TIP = u"Tell Envoy to submit your job without uploading any new files."

    def __init__(self, plugin, packed_project, parent=None):
        QtWidgets.QDialog.__init__(self, parent)
        QtPluginAccessor.__init__(self, plugin)

        self.setWindowModality(QtCore.Qt.ApplicationModal)

        # apply style sheet
        plugin.get_user_interface().set_style_sheet(self)
        self.setProperty(qt_constants.STYLE_PROPS.BACKGROUND_LEVEL_1, True)

        # Remove the window help button that only appears on Windows OS
        self.setWindowFlags(self.windowFlags() ^ QtCore.Qt.WindowContextHelpButtonHint)

        # Set the window title to the plugin name
        self.setWindowTitle(u"GridMarkets Preflight")

        # resize the window to it's inital size
        self.resize(PreflightWindow.INITIAL_WINDOW_WIDTH, PreflightWindow.INITIAL_WINDOW_HEIGHT)

        # set minimum dimensions
        self.setMinimumWidth(PreflightWindow.MINIMUM_WINDOW_WIDTH)
        self.setMinimumHeight(PreflightWindow.MINIMUM_WINDOW_HEIGHT)

        self.setWindowIcon(plugin.get_user_interface().get_icon())

        self._cancel_submission = True

        """ Create Widgets """
        self._header_layout_widget = QtWidgets.QWidget()
        self._header_layout_widget.setProperty(qt_constants.STYLE_PROPS.BACKGROUND_LEVEL_0, True)

        self._title = QtWidgets.QLabel(u"Preflight")
        self._title.setProperty(qt_constants.STYLE_PROPS.H3, True)
        self._preflight_disclaimer = QtWidgets.QLabel(u"Please select all files you wish to upload.")

        self.file_tree_model = FileTreeModel(packed_project)

        self._tree_view = QtWidgets.QTreeView()
        self._tree_view.setModel(self.file_tree_model)
        self.file_tree_model.set_default_expanded_state(self._tree_view)
        self._tree_view.header().setStretchLastSection(False)
        self._tree_view.header().setSectionResizeMode(0, QtWidgets.QHeaderView.Stretch)
        self._tree_view.setAlternatingRowColors(True)

        self._controls_layout_widget = QtWidgets.QWidget()

        self._cancel_btn = QtWidgets.QPushButton(u"Cancel")

        self._primary_action_1 = QtWidgets.QAction(PreflightWindow.UPLOAD_AND_SUBMIT, self)
        self._primary_action_1.setToolTip(PreflightWindow.UPLOAD_AND_SUBMIT_TOOL_TIP)
        self._primary_action_2 = QtWidgets.QAction(PreflightWindow.UPLOAD, self)
        self._primary_action_2.setToolTip(PreflightWindow.UPLOAD_TOOL_TIP)
        self._primary_action_3 = QtWidgets.QAction(PreflightWindow.SUBMIT, self)
        self._primary_action_3.setToolTip(PreflightWindow.SUBMIT_TOOL_TIP)

        self._primary_actions_menu = QtWidgets.QMenu()
        self._primary_actions_menu.setToolTipsVisible(True)
        self._primary_actions_menu.addAction(self._primary_action_1)
        self._primary_actions_menu.addAction(self._primary_action_2)

        # only show the submit action if the project already exists
        if packed_project.get_name() in plugin.get_api_client().get_root_directories():
            self._primary_actions_menu.addAction(self._primary_action_3)

        plugin.get_user_interface().set_style_sheet(self._primary_actions_menu)

        self._primary_btn = QtWidgets.QToolButton()
        self._primary_btn.setProperty(qt_constants.STYLE_PROPS.BRAND, True)
        self._primary_btn.setMinimumWidth(175)
        self._primary_btn.setMenu(self._primary_actions_menu)
        self._primary_btn.setPopupMode(QtWidgets.QToolButton.MenuButtonPopup)
        self.primary_action_1_slot()

        """ Create Connections """
        self._cancel_btn.clicked.connect(self.cancel)
        self._primary_btn.clicked.connect(self.submit)
        self._primary_actions_menu.aboutToShow.connect(self._about_to_show_primary_actions_menu)

        self._primary_action_1.triggered.connect(self.primary_action_1_slot)
        self._primary_action_2.triggered.connect(self.primary_action_2_slot)
        self._primary_action_3.triggered.connect(self.primary_action_3_slot)

        """ Create Layouts """
        header_layout = QtWidgets.QVBoxLayout()
        header_layout.addWidget(self._title)
        self._header_layout_widget.setLayout(header_layout)

        body_layout = QtWidgets.QVBoxLayout()
        body_layout.addWidget(self._preflight_disclaimer)
        body_layout.addSpacing(10)
        body_layout.addWidget(self._tree_view)
        body_layout.setContentsMargins(10,10,10,10)

        controls_layout = QtWidgets.QHBoxLayout()
        controls_layout.setContentsMargins(0,0,0,0)
        controls_layout.setAlignment(QtCore.Qt.AlignRight)
        controls_layout.addWidget(self._primary_btn)
        controls_layout.addWidget(self._cancel_btn)
        self._controls_layout_widget.setLayout(controls_layout)

        layout = QtWidgets.QVBoxLayout(self)
        layout.addWidget(self._header_layout_widget)
        layout.addLayout(body_layout)
        layout.addWidget(self._controls_layout_widget)

    def primary_action_1_slot(self):
        self._primary_btn.setText(PreflightWindow.UPLOAD_AND_SUBMIT)
        self._primary_btn.setToolTip(PreflightWindow.UPLOAD_AND_SUBMIT_TOOL_TIP)

    def primary_action_2_slot(self):
        self._primary_btn.setText(PreflightWindow.UPLOAD)
        self._primary_btn.setToolTip(PreflightWindow.UPLOAD_TOOL_TIP)

    def primary_action_3_slot(self):
        self._primary_btn.setText(PreflightWindow.SUBMIT)
        self._primary_btn.setToolTip(PreflightWindow.SUBMIT_TOOL_TIP)

    def cancel(self):
        self._cancel_submission = True
        self.hide()

    def submit(self):
        files_to_upload = self.file_tree_model.get_files_to_upload()
        self._cancel_submission = False
        self.hide()

    def _about_to_show_primary_actions_menu(self):
        self._primary_actions_menu.setFixedWidth(self._primary_btn.width())

    def cancel_submission(self):
        return self._cancel_submission

    def get_next_action(self):
        return self._primary_btn.text()

    def get_files_to_upload(self):
        return self.file_tree_model.get_files_to_upload()
