__all__ = 'JobPresetContainer'

from functools import partial

from PySide2 import QtCore
from PySide2 import QtGui
from PySide2 import QtWidgets

import gm_meta_plugin
from gm_meta_plugin.gridmarkets import constants as api_constants
from gm_qt_plugin import qt_constants, qt_utils
from gm_qt_plugin.widgets.job_preset import JobPreset
from gm_qt_plugin.widgets.qhline import QHLine


class JobPresetContainer(QtWidgets.QWidget, gm_meta_plugin.JobPresetContainer):

    def __init__(self, plugin, log_items, parent=None):
        QtWidgets.QWidget.__init__(self, parent=parent)
        gm_meta_plugin.JobPresetContainer.__init__(self, plugin, log_items)

        """ Create Widgets """
        self.title_label = QtWidgets.QLabel(u"Job Presets")
        self.title_label.setProperty("h2", True)

        self.qhline = QHLine()

        self.job_preset_options_menu = QtWidgets.QMenu()
        self.job_preset_options_menu.aboutToShow.connect(self.about_to_show_job_preset_menu)

        self.job_preset_options_drop_down_btn = QtWidgets.QPushButton(qt_constants.NEW_JOB_PRESET_ICON_WHITE,
                                                                      u"Create a new Job Preset")
        self.job_preset_options_drop_down_btn.setMenu(self.job_preset_options_menu)

        self._show_all_action = QtWidgets.QAction("Show all job definition types", self)
        self._show_all_action.setCheckable(True)
        self._show_all_action.setChecked(True)

        self.delete_focused_job_preset_btn = QtWidgets.QPushButton(qt_constants.TRASH_ICON_BLACK, u"")
        self.delete_focused_job_preset_btn.setFlat(True)
        self.delete_focused_job_preset_btn.setMaximumWidth(32)
        self.delete_focused_job_preset_btn.clicked.connect(self.remove_focused)

        self.lock_focused_job_preset_btn = QtWidgets.QPushButton(qt_constants.LOCKED_ICON_BLACK, u"")
        self.lock_focused_job_preset_btn.setFlat(True)
        self.lock_focused_job_preset_btn.setMaximumWidth(32)
        self.lock_focused_job_preset_btn.clicked.connect(self.toggle_lock_focused_job_preset)

        self.job_preset_options_settings_menu = QtWidgets.QMenu()
        self.job_preset_options_settings_menu.addAction(self._show_all_action)
        self.job_preset_options_settings_btn = QtWidgets.QPushButton(qt_constants.MENU_ICON_WHITE, u"")
        self.job_preset_options_settings_btn.setMenu(self.job_preset_options_settings_menu)
        self.job_preset_options_settings_btn.setMaximumWidth(32)
        self.job_preset_options_settings_btn.setProperty('noMenuIndicator', True)

        self.job_presets_list = QtWidgets.QTreeWidget()
        self.job_presets_list.setColumnCount(2)
        self.job_presets_list.header().setStretchLastSection(False)
        self.job_presets_list.header().setSectionResizeMode(0, QtWidgets.QHeaderView.Stretch)
        self.job_presets_list.setHeaderHidden(True)
        self.job_presets_list.setColumnWidth(0, 200)
        self.job_presets_list.setColumnWidth(1, 20)
        self.job_presets_list.setRootIsDecorated(False)
        self.job_presets_list.itemDoubleClicked.connect(self.list_item_double_clicked)
        self.job_presets_list.setMaximumHeight(120)
        self.job_presets_list.currentItemChanged.connect(self.current_item_changed)
        self.job_presets_list.itemChanged.connect(self.list_item_changed)

        """ Create Layouts """
        self.create_layouts()

    def create_layouts(self):
        self.setSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)

        # job preset options layout
        self.job_preset_options_layout = QtWidgets.QHBoxLayout()
        self.job_preset_options_layout.setSpacing(0)
        self.job_preset_options_layout.addWidget(self.job_preset_options_drop_down_btn)
        self.job_preset_options_layout.addWidget(self.job_preset_options_settings_btn)

        # job preset list controls layout
        self.list_controls_layout = QtWidgets.QVBoxLayout()
        self.list_controls_layout.setAlignment(QtCore.Qt.AlignTop)

        self.list_controls_layout.addWidget(self.delete_focused_job_preset_btn)
        self.list_controls_layout.addWidget(self.lock_focused_job_preset_btn)

        # job preset list layout
        self.job_preset_list_layout = QtWidgets.QHBoxLayout()
        self.job_preset_list_layout.addWidget(self.job_presets_list)
        self.job_preset_list_layout.addLayout(self.list_controls_layout)

        # temporary layout for showing the job preset settings in
        self.job_preset_layout = QtWidgets.QVBoxLayout()

        self.job_preset_container_layout = QtWidgets.QVBoxLayout(self)
        self.job_preset_container_layout.addLayout(self.job_preset_options_layout)
        self.job_preset_container_layout.addLayout(self.job_preset_list_layout)

        group_box = QtWidgets.QGroupBox()
        group_box.setLayout(self.job_preset_container_layout)

        self.title_layout = QtWidgets.QHBoxLayout()
        self.title_layout.setAlignment(QtCore.Qt.AlignLeft)
        self.title_layout.addWidget(self.title_label)

        self.job_presets_layout = QtWidgets.QVBoxLayout(self)
        self.job_presets_layout.setContentsMargins(0, 0, 0, 0)
        self.job_presets_layout.addLayout(self.title_layout)
        self.job_presets_layout.addWidget(self.qhline)
        self.job_presets_layout.addWidget(group_box)
        self.job_presets_layout.addLayout(self.job_preset_layout)

    def list_item_changed(self, item, col):
        job_preset = self.convert_item_to_preset(item)

        # update the name of the preset
        if col == 0:
            job_preset.set_name(item.text(col))

    def list_item_double_clicked(self, item):
        self.job_presets_list.editItem(item)

    def convert_item_to_preset(self, item):
        return item.data(0, QtCore.Qt.UserRole)

    def convert_items_to_presets(self, items):
        return list(map(lambda item: self.convert_item_to_preset(item), items))

    def update_job_preset_layout(self):
        # only update if the job preset container is the current widget in the main_ui_switcher, otherwise by redrawing
        # the job presets layout you may remove it from the current viewed widget (since they share the widgets)

        main_switcher_ui = self.get_plugin().get_user_interface().get_main_switcher_ui()

        if main_switcher_ui.get_current_widget() == self:
            self.draw_focused_job_preset()

    def current_item_changed(self, current, previous):
        job_preset = None if current is None else self.convert_item_to_preset(current)
        previous_job_preset = None if previous is None else self.convert_item_to_preset(previous)

        # set the enabled state of the control buttons
        enabled = current is not None
        self.delete_focused_job_preset_btn.setEnabled(enabled)
        self.lock_focused_job_preset_btn.setEnabled(enabled)

        # disconnect all signals from the previous job preset
        if previous_job_preset is not None:
            self.lock_focused_job_preset_btn.clicked.disconnect()

        # set the lock icon on list controls btn
        if job_preset is not None:
            icon = qt_constants.UNLOCKED_ICON_WHITE if job_preset.is_locked() else qt_constants.LOCKED_ICON_WHITE
            self.lock_focused_job_preset_btn.setIcon(icon)
            self.lock_focused_job_preset_btn.clicked.connect(job_preset.toggle_locked_state)
            self.focus_item(job_preset)

        # draw the layout
        self.draw_focused_job_preset()

    def draw_focused_job_preset(self):
        # remove the old layout from the main layout
        #self.job_presets_layout.removeItem(self.job_preset_layout)

        # remove widgets from the old layout layout (otherwise they still display)
        #qt_utils.remove_widgets_from_layout(self.job_preset_layout)

        # get the focused job preset
        focused_item = self.job_presets_list.currentItem()
        if focused_item:
            job_preset = self.convert_item_to_preset(focused_item)
        else:
            job_preset = None

        # build the new layout
        if job_preset:
            self.job_preset_layout.addWidget(job_preset)

        # display the new layout
        #self.job_presets_layout.addLayout(self.job_preset_layout)
        #qt_utils.show_all_widgets_in_layout(self.job_preset_layout)

    def toggle_lock_focused_job_preset(self):
        """ Redraws the lock icon for the job preset tree item so that it matched the job presets locked state """
        focused_item = self.job_presets_list.currentItem()

        if focused_item is None:
            return

        job_preset = self.convert_item_to_preset(focused_item)
        self.re_draw_job_preset_list_item(job_preset)

    def re_draw_job_preset_list_item(self, job_preset):

        if job_preset is None:
            return

        job_preset_item = self.job_preset_to_tree_item(job_preset)

        if job_preset.is_locked():
            job_preset_item.setFlags(job_preset_item.flags() & ~QtCore.Qt.ItemIsEditable)
            foreground_color = QtGui.QColor(180, 180, 180, 100)  # todo remove hard coded colors
            job_preset_item.setIcon(1, qt_constants.LOCKED_ICON_WHITE)
            self.lock_focused_job_preset_btn.setIcon(qt_constants.UNLOCKED_ICON_WHITE)
        else:
            job_preset_item.setFlags(job_preset_item.flags() | QtCore.Qt.ItemIsEditable)
            foreground_color = QtGui.QColor(180, 180, 180, 255)
            job_preset_item.setIcon(1, qt_constants.UNLOCKED_ICON_WHITE)
            self.lock_focused_job_preset_btn.setIcon(qt_constants.LOCKED_ICON_WHITE)

        job_preset_item.setForeground(0, QtGui.QBrush(foreground_color))

    def about_to_show_job_preset_menu(self):
        # set menu width to be the same as the button width
        btn_width = self.job_preset_options_drop_down_btn.width()
        self.job_preset_options_menu.setMinimumWidth(btn_width)

        # remove all items from the menu
        self.job_preset_options_menu.clear()

        api_schema = self.get_plugin().get_api_client().get_cached_api_schema()

        if api_schema is not None:
            for job_definition in api_schema.get_job_definitions():

                job_definition_app_attribute = job_definition.get_attribute_with_key(api_constants.API_KEYS.APP)
                job_definition_app = job_definition_app_attribute.get_attribute().get_default_value()

                if not self._show_all_action.isChecked():  # show only job definitions pertaining to maya
                    if job_definition_app != api_constants.PRODUCTS.MAYA:
                        continue

                action = QtWidgets.QAction(job_definition.get_display_name(), self)
                action.setIcon(qt_utils.get_product_icon(job_definition_app))
                action.triggered.connect(partial(self.add_job_preset_slot, job_definition.get_definition_id()))
                self.job_preset_options_menu.addAction(action)

    def add_job_preset_slot(self, job_definition_id):
        plugin = self.get_plugin()
        api_schema = self.get_plugin().get_api_client().get_cached_api_schema()
        user_interface = plugin.get_user_interface()
        logging_coordinator = plugin.get_logging_coordinator()
        logger = logging_coordinator.get_logger(__name__)

        job_definition = api_schema.get_job_definition_with_id(job_definition_id)

        # get the job preset objects in the QListWidgetItems
        job_preset_items = list(
            self.convert_item_to_preset(self.job_presets_list.topLevelItem(i)) for i in
            range(self.job_presets_list.topLevelItemCount()))

        name_prefix = (job_definition.get_display_name() + "_Job_Preset_").replace(' ', '_')

        # create a unique id
        id = gm_meta_plugin.utils.create_unique_object_name(job_preset_items,
                                                            attribute_name="get_id",
                                                            name_prefix=name_prefix)

        # create a unique name
        name = gm_meta_plugin.utils.create_unique_object_name(job_preset_items,
                                                              attribute_name="get_name",
                                                              name_prefix=name_prefix)

        logger.info(u"Creating Job Preset...")

        worker_thread = user_interface.get_worker_thread(u"Creating job preset...")

        ################################################################################################################

        target = JobPreset

        args = (
            self.get_plugin(),
            name,
            id,
            job_definition,
            self.re_draw_job_preset_list_item
        )

        kwargs = {
            'is_locked': False
        }

        worker_thread.process(target, args, kwargs, self._handle_append_job_preset_result)


    def _handle_append_job_preset_result(self, result):
        if isinstance(result, JobPreset):
            """
            This is very hacky but it works.
            
            In Qt it is technically an error to create a QWidget in any thread other than the Main GUI thread. This 
            means we can not use the job preset just created because it was created in a different thread. However
            because all the long running parts of creating a Job Preset get cached it is very quick to re-create the 
            same job preset here in the Gui thread and use the new version instead. 
            """
            job_preset = JobPreset(self.get_plugin(),
                                   result.get_name(),
                                   result.get_id(),
                                   result.get_job_definition(),
                                   self.re_draw_job_preset_list_item,
                                   is_locked=False)

            self.append(job_preset, focus_new_item=True)
            return True
        return False

    def job_preset_to_tree_item(self, job_preset):
        i = 0
        count = self.job_presets_list.topLevelItemCount()
        item = None
        while i < count:
            item = self.job_presets_list.topLevelItem(i)
            item_preset = self.convert_item_to_preset(item)
            if item_preset == job_preset:
                break

            i = i + 1

        return item

    def focus_item(self, job_preset, clear_selection=True):
        gm_meta_plugin.JobPresetContainer.focus_item(self, job_preset, clear_selection=clear_selection)
        item = self.job_preset_to_tree_item(job_preset)
        self.job_presets_list.setCurrentItem(item)

    def append(self, job_preset, focus_new_item=True):

        item = QtWidgets.QTreeWidgetItem(job_preset.get_name())
        item.setText(0, job_preset.get_name())
        item.setIcon(0, job_preset.get_icon())
        item.setText(1, '')
        item.setIcon(1, qt_constants.UNLOCKED_ICON_WHITE)
        item.setTextAlignment(1, QtCore.Qt.AlignRight)
        item.setFlags(item.flags() | QtCore.Qt.ItemIsEditable)
        item.setData(0, QtCore.Qt.UserRole, job_preset)

        self.job_presets_list.addTopLevelItem(item)

        gm_meta_plugin.JobPresetContainer.append(self, job_preset, focus_new_item=focus_new_item)

    def remove(self, job_preset):
        gm_meta_plugin.JobPresetContainer.remove(self, job_preset)
        item = self.job_preset_to_tree_item(job_preset)
        index = self.job_presets_list.indexOfTopLevelItem(item)
        self.job_presets_list.takeTopLevelItem(index)
