import gm_meta_plugin
from gm_meta_plugin.gridmarkets import constants as api_constants

from gm_qt_plugin import QtPlugin, QtPreferencesContainer, QtAPISchemaFactory, QtProjectAttributeInstanceFactory, \
    QtProjectInstanceBuilderFactory
from gm_qt_plugin.widgets import JobPresetContainer, LogHistoryContainer, RemoteProjectContainer, JobPreset

from .api_client import APIClient
from .plugin_utils import PluginUtils
from .maya_attribute_source import MayaAttributeSource
from .packed_scene_builder import PackedSceneBuilder
from .maya_user_interface import MayaUserInterface
from . import maya_utils
from .job_preset_validator import JobPresetValidator


class MayaPlugin(QtPlugin):

    def _get_factory_collection(self):
        api_schema_factory = QtAPISchemaFactory()
        project_attribute_factory = gm_meta_plugin.ProjectAttributeFactory(self)
        project_attribute_instance_factory = QtProjectAttributeInstanceFactory(self)
        project_instance_builder_factory = QtProjectInstanceBuilderFactory(self)

        return gm_meta_plugin.FactoryCollection(api_schema_factory,
                                                project_attribute_factory,
                                                project_attribute_instance_factory,
                                                project_instance_builder_factory)

    def __init__(self):
        # remember to keep in sync with the version in the maya2gm.py file
        version = (1, 2, 2)

        main_window = maya_utils.maya_main_window()

        self._version = gm_meta_plugin.PluginVersion(version[0], version[1], version[2])

        self._factory_collection = self._get_factory_collection()

        log_history_container = LogHistoryContainer(self, [])
        self._logging_coordinator = gm_meta_plugin.LoggingCoordinator(self, log_history_container)

        user_container = gm_meta_plugin.UserContainer(self, [])
        job_preset_container = JobPresetContainer(self, [], parent=main_window)
        self._preferences_container = QtPreferencesContainer(user_container, job_preset_container)

        self._api_client = APIClient(self, self._logging_coordinator, self._factory_collection)
        self._user_interface = MayaUserInterface(self)
        self._remote_project_container = RemoteProjectContainer(self, [], parent=main_window)
        self._plugin_utils = PluginUtils()
        self._application_attribute_source = MayaAttributeSource(self)
        self._packed_scene_builder = PackedSceneBuilder(self)
        self._job_preset_validator = JobPresetValidator(self)

    def get_name(self):
        return "Maya2GM"

    def get_version(self):
        return self._version

    def get_logging_coordinator(self):
        return self._logging_coordinator

    def get_preferences_container(self):
        return self._preferences_container

    def get_api_client(self):
        return self._api_client

    def get_user_interface(self):
        return self._user_interface

    def get_remote_project_container(self):
        return self._remote_project_container

    def get_application_attribute_source(self):
        return self._application_attribute_source

    def get_packed_scene_builder(self):
        return self._packed_scene_builder

    def get_factory_collection(self):
        return self._factory_collection

    def get_plugin_utils(self):
        return self._plugin_utils

    def get_supported_applications(self):
        return [api_constants.PRODUCTS.MAYA]

    def create_default_job_presets(self):
        api_schema = self.get_api_client().get_cached_api_schema()
        job_preset_container = self._preferences_container.get_job_preset_container()

        job_definition = api_schema.get_job_definition_with_id(api_constants.JOB_DEFINITION_IDS.MAYA)

        # get the job preset objects in the QListWidgetItems
        job_preset_items = job_preset_container.get_all()

        name_prefix = (job_definition.get_display_name() + "_Job_Preset_").replace(' ', '_')

        # create a unique id
        id = gm_meta_plugin.utils.create_unique_object_name(job_preset_items,
                                                            attribute_name="get_id",
                                                            name_prefix=name_prefix)

        name = "Use Maya's current render settings"

        default_job_preset = JobPreset(self,
                                       name,
                                       id,
                                       job_definition,
                                       job_preset_container.re_draw_job_preset_list_item)

        job_preset_container.append(default_job_preset)

    def get_job_preset_validator(self):
        return self._job_preset_validator
